#!/usr/bin/python
# -*- coding: utf-8 -*-

import pgm
from pypgmtools.graph.group import Group
from pypgmtools.graph.image import Image

import glob

# Constants for navigation metaphor
LEFT_RIGHT = 0
TOP_BOTTOM = 1

class WidgetGridImage(Group):
    def __init__(self, canvas, images,
                 width, height, cols, rows,
                 navigation=LEFT_RIGHT, visible_cols=None, visible_rows=None):
        Group.__init__(self, canvas)
        self._width = float(width)
        self._height = float(height)
        self._cols = cols
        self._rows = rows
        # Truncate the list of images to the maximum number of displayable elements
        self._images = images[:(self._cols * self._rows)]
        self._navigation = navigation
        # By default, show all the columns and rows
        self._visible_cols = cols
        self._visible_rows = rows
        if (navigation == LEFT_RIGHT) and (visible_cols is not None) and (visible_cols < cols):
            # In left-right mode, we can have less columns displayed
            # than the actual total number of columns
            self._visible_cols = visible_cols
        if (navigation == TOP_BOTTOM) and (visible_rows is not None) and (visible_rows < rows):
            # In top-bottom mode, we can have less rows displayed
            # than the actual total number of rows
            self._visible_rows = visible_rows
        print 'Visible columns:', str(self._visible_cols)
        print 'Visible rows:', str(self._visible_rows)

        # Compute the size of the elements (squares) and the spacing between them
        # The spacing factor is the ratio between the spacing and the width of an element
        self._spacing_factor = 0.5
        # The residual factor is the ratio between the part of the elements
        # shown on the sides and the width of an element
        self._residual_factor = 0.3
        a = self._width / ((self._visible_cols + 1) * self._spacing_factor + self._visible_cols + 2 * self._residual_factor)
        b = self._height / ((self._visible_rows + 1) * self._spacing_factor + self._visible_rows + 2 * self._residual_factor)
        self._elem_size = (a, b)
        print 'Element size:', str(self._elem_size)

        # TODO: take into account the current selection to correctly display the elements
        #self._selection = (self._visible_cols / 2, self._visible_rows / 2)

        current_col = 1
        current_row = 1
        for image in self._images:
            image.size = self._elem_size
            image.visible = True
            image_x = self._residual_factor * self._elem_size[0] + \
                      self._spacing_factor * self._elem_size[0] + \
                      self._elem_size[0] * (1 + self._spacing_factor) * (current_col - 1)
            image_y = self._residual_factor * self._elem_size[1] + \
                      self._spacing_factor * self._elem_size[1] + \
                      self._elem_size[1] * (1 + self._spacing_factor) * (current_row - 1)
            image.position = (image_x, image_y, 0.0)
            self.add(image)
            print 'Added image on the grid at position', str(image.position[0]), ',', str(image.position[1])
            if (self._navigation == LEFT_RIGHT):
                current_col += 1
                if current_col > self._cols:
                    current_col = 1
                    current_row += 1
            elif (self._navigation == TOP_BOTTOM):
                current_row += 1
                if current_row > self._rows:
                    current_row = 1
                    current_col += 1

# Terminate the mainloop on a delete event
def on_delete(viewport, event):
    pgm.main_quit()

# Update the viewport on an expose event
def on_expose(viewport, event):
    viewport.update()

def on_key_press(viewport, event):
    # Terminate the mainloop when the ESC key or the 'q' key is pressed
    if (event.keyval == pgm.keysyms.q) or (event.keyval == pgm.keysyms.Escape):
        pgm.main_quit()

if __name__ == '__main__':
    # OpenGL viewport creation
    factory = pgm.ViewportFactory('opengl')
    viewport = factory.create()
    viewport.title = 'WidgetImage test'

    # Canvas creation
    canvas = pgm.Canvas()
    canvas.size = (800.0, 600.0)

    # Bind the canvas to the OpenGL viewport
    viewport.set_canvas(canvas)

    viewport.connect('delete-event', on_delete)
    viewport.connect('expose-event', on_expose)
    viewport.connect('key-press-event', on_key_press)

    # List of images
    files = glob.glob('covers/*.jpg')
    images = []
    for f in files:
        image = Image()
        image.set_from_file(f)
        images.append(image)

    widget = WidgetGridImage(canvas, images, 800, 600, 5, 4, LEFT_RIGHT, 5, 4)
    widget.position = (0.0, 0.0, 0.0)
    widget.visible = True

    # show the window
    viewport.show()
    viewport.update()

    # Main loop
    pgm.main()

